@@ -17,15 +17,16 @@ import logging
 def positive_float(s: str) -> float:
     x = float(s)
     if not x > 0:
-        raise argparse.ArgumentTypeError("%r is not a positive number" % s)
+        raise argparse.ArgumentTypeError(f"{s!r} is not a positive number")
     return x
 
+
 def character_encoding(s: str) -> str:
     try:
         codecs.lookup(s)
         return s
     except LookupError:
-        raise argparse.ArgumentError(None, "unknown character encoding: {}".format(s))
+        raise argparse.ArgumentError(None, f"unknown character encoding: {s}")
 
 
 def time(s: str) -> int:
@@ -63,7 +64,7 @@ class Pysubs2CLI:
                                  "MicroDVD (*.sub) or other supported format. When no files are specified, "
                                  "pysubs2 will work as a pipe, reading from standard input and writing to standard output.")
 
-        parser.add_argument("-v", "--version", action="version", version="pysubs2 %s" % VERSION)
+        parser.add_argument("-v", "--version", action="version", version=f"pysubs2 {VERSION}")
 
         parser.add_argument("-f", "--from", choices=FORMAT_IDENTIFIERS, dest="input_format",
                             help="By default, subtitle format is detected from the file. This option can be used to "
@@ -103,11 +104,17 @@ class Pysubs2CLI:
                            help="Multiply all timestamps by FPS1/FPS2 ratio.")
 
         extra_srt_options = parser.add_argument_group("optional arguments (SRT)")
-        extra_srt_options.add_argument("--srt-keep-unknown-html-tags", action="store_true")
-        extra_srt_options.add_argument("--srt-keep-ssa-tags", action="store_true")
+        extra_srt_options.add_argument("--srt-keep-unknown-html-tags", action="store_true",
+                                       help="(input) do not strip unrecognized HTML tags")
+        extra_srt_options.add_argument("--srt-keep-html-tags", action="store_true",
+                                       help="(input) do not convert HTML tags to SubStation internally,"
+                                            " this implies --srt-keep-unknown-html-tags")
+        extra_srt_options.add_argument("--srt-keep-ssa-tags", action="store_true",
+                                       help="(output) do not convert/strip SubStation tags for output")
 
         extra_sub_options = parser.add_argument_group("optional arguments (MicroDVD)")
-        extra_sub_options.add_argument("--sub-no-write-fps-declaration", action="store_true")
+        extra_sub_options.add_argument("--sub-no-write-fps-declaration", action="store_true",
+                                       help="(output) omit writing FPS as first zero-length subtitle")
 
     def __call__(self, argv):
         try:
@@ -132,6 +139,8 @@ class Pysubs2CLI:
         extra_output_args = {}
         if args.srt_keep_unknown_html_tags:
             extra_input_args["keep_unknown_html_tags"] = True
+        if args.srt_keep_html_tags:
+            extra_input_args["keep_html_tags"] = True
         if args.srt_keep_ssa_tags:
             extra_output_args["keep_ssa_tags"] = True
         if args.sub_no_write_fps_declaration:
