@@ -12,13 +12,16 @@ MICRODVD_LINE = re.compile(r" *\{ *(\d+) *\} *\{ *(\d+) *\}(.+)")
 
 
 class MicroDVDFormat(FormatBase):
+    """MicroDVD subtitle format implementation"""
     @classmethod
     def guess_format(cls, text):
+        """See :meth:`pysubs2.formats.FormatBase.guess_format()`"""
         if any(map(MICRODVD_LINE.match, text.splitlines())):
             return "microdvd"
 
     @classmethod
     def from_file(cls, subs, fp, format_, fps=None, **kwargs):
+        """See :meth:`pysubs2.formats.FormatBase.from_file()`"""
         for line in fp:
             match = MICRODVD_LINE.match(line)
             if not match:
@@ -45,9 +48,9 @@ class MicroDVDFormat(FormatBase):
             def prepare_text(text):
                 text = text.replace("|", r"\N")
 
-                def style_replacer(match):
+                def style_replacer(match: re.Match) -> str:
                     tags = [c for c in "biu" if c in match.group(0)]
-                    return "{%s}" % "".join(r"\%s1" % c for c in tags)
+                    return "{%s}" % "".join(f"\\{c}1" for c in tags)
 
                 text = re.sub(r"\{[Yy]:[^}]+\}", style_replacer, text)
                 text = re.sub(r"\{[Ff]:([^}]+)\}", r"{\\fn\1}", text)
@@ -60,7 +63,18 @@ class MicroDVDFormat(FormatBase):
             subs.append(ev)
 
     @classmethod
-    def to_file(cls, subs, fp, format_, fps=None, write_fps_declaration=True, **kwargs):
+    def to_file(cls, subs, fp, format_, fps=None, write_fps_declaration=True, apply_styles=True, **kwargs):
+        """
+        See :meth:`pysubs2.formats.FormatBase.to_file()`
+
+        The only supported styling is marking whole lines italic.
+
+        Keyword args:
+            write_fps_declaration: If True, create a zero-duration first subtitle which will contain
+                the fps.
+            apply_styles: If False, do not write any styling.
+
+        """
         if fps is None:
             fps = subs.fps
 
@@ -68,7 +82,7 @@ class MicroDVDFormat(FormatBase):
             raise UnknownFPSError("Framerate must be specified when writing MicroDVD.")
         to_frames = partial(ms_to_frames, fps=fps)
 
-        def is_entirely_italic(line):
+        def is_entirely_italic(line: SSAEvent) -> bool:
             style = subs.styles.get(line.style, SSAStyle.DEFAULT_STYLE)
             for fragment, sty in parse_tags(line.text, style, subs.styles):
                 fragment = fragment.replace(r"\h", " ")
@@ -87,7 +101,7 @@ class MicroDVDFormat(FormatBase):
                 continue
 
             text = "|".join(line.plaintext.splitlines())
-            if is_entirely_italic(line):
+            if apply_styles and is_entirely_italic(line):
                 text = "{Y:i}" + text
 
             start, end = map(to_frames, (line.start, line.end))
