@@ -1,10 +1,10 @@
-from collections import MutableSequence
+from collections.abc import MutableSequence
 import io
 from io import open
 from itertools import chain
 import os.path
 import logging
-from typing import Optional, List, Dict, Iterable, Any
+from typing import Optional, List, Dict, Iterable, Any, overload, Iterator
 
 from .common import IntOrFloat
 from .formats import autodetect_format, get_format_class, get_format_identifier
@@ -38,12 +38,13 @@ class SSAFile(MutableSequence):
         "Collisions": "Normal"
     }
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.events: List[SSAEvent] = []  #: List of :class:`SSAEvent` instances, ie. individual subtitles.
         self.styles: Dict[str, SSAStyle] = {"Default": SSAStyle.DEFAULT_STYLE.copy()}  #: Dict of :class:`SSAStyle` instances.
         self.info: Dict[str, str] = self.DEFAULT_INFO.copy()  #: Dict with script metadata, ie. ``[Script Info]``.
         self.aegisub_project: Dict[str, str] = {}  #: Dict with Aegisub project, ie. ``[Aegisub Project Garbage]``.
         self.fonts_opaque: Dict[str, Any] = {}  #: Dict with embedded fonts, ie. ``[Fonts]``.
+        self.graphics_opaque: Dict[str, Any] = {}  #: Dict with embedded images, ie. ``[Graphics]``.
         self.fps: Optional[float] = None  #: Framerate used when reading the file, if applicable.
         self.format: Optional[str] = None  #: Format of source subtitle file, if applicable, eg. ``"srt"``.
 
@@ -282,7 +283,7 @@ class SSAFile(MutableSequence):
 
         """
         if in_fps <= 0 or out_fps <= 0:
-            raise ValueError("Framerates must be positive, cannot transform %f -> %f" % (in_fps, out_fps))
+            raise ValueError(f"Framerates must be positive, cannot transform {in_fps} -> {out_fps}")
 
         ratio = in_fps / out_fps
         for line in self:
@@ -308,11 +309,11 @@ class SSAFile(MutableSequence):
 
         """
         if old_name not in self.styles:
-            raise KeyError("Style %r not found" % old_name)
+            raise KeyError(f"Style {old_name!r} not found")
         if new_name in self.styles:
-            raise ValueError("There is already a style called %r" % new_name)
+            raise ValueError(f"There is already a style called {new_name!r}")
         if not is_valid_field_content(new_name):
-            raise ValueError("%r is not a valid name" % new_name)
+            raise ValueError(f"{new_name!r} is not a valid name")
 
         self.styles[new_name] = self.styles[old_name]
         del self.styles[old_name]
@@ -389,52 +390,64 @@ class SSAFile(MutableSequence):
 
         if isinstance(other, SSAFile):
             for key in set(chain(self.info.keys(), other.info.keys())) - {"ScriptType"}:
-                sv, ov = self.info.get(key), other.info.get(key)
-                if sv is None:
+                self_info, other_info = self.info.get(key), other.info.get(key)
+                if self_info is None:
                     logging.debug("%r missing in self.info", key)
                     return False
-                elif ov is None:
+                elif other_info is None:
                     logging.debug("%r missing in other.info", key)
                     return False
-                elif sv != ov:
-                    logging.debug("info %r differs (self=%r, other=%r)", key, sv, ov)
+                elif self_info != other_info:
+                    logging.debug("info %r differs (self=%r, other=%r)", key, self_info, other_info)
                     return False
 
             for key in set(chain(self.fonts_opaque.keys(), other.fonts_opaque.keys())):
-                sv, ov = self.fonts_opaque.get(key), other.fonts_opaque.get(key)
-                if sv is None:
+                self_font, other_font = self.fonts_opaque.get(key), other.fonts_opaque.get(key)
+                if self_font is None:
                     logging.debug("%r missing in self.fonts_opaque", key)
                     return False
-                elif ov is None:
+                elif other_font is None:
                     logging.debug("%r missing in other.fonts_opaque", key)
                     return False
-                elif sv != ov:
-                    logging.debug("fonts_opaque %r differs (self=%r, other=%r)", key, sv, ov)
+                elif self_font != other_font:
+                    logging.debug("fonts_opaque %r differs (self=%r, other=%r)", key, self_font, other_font)
+                    return False
+
+            for key in set(chain(self.graphics_opaque.keys(), other.graphics_opaque.keys())):
+                self_image, other_image = self.graphics_opaque.get(key), other.graphics_opaque.get(key)
+                if self_image is None:
+                    logging.debug("%r missing in self.graphics_opaque", key)
+                    return False
+                elif other_image is None:
+                    logging.debug("%r missing in other.graphics_opaque", key)
+                    return False
+                elif self_image != other_image:
+                    logging.debug("graphics_opaque %r differs (self=%r, other=%r)", key, self_image, other_image)
                     return False
 
             for key in set(chain(self.styles.keys(), other.styles.keys())):
-                sv, ov = self.styles.get(key), other.styles.get(key)
-                if sv is None:
+                self_style, other_style = self.styles.get(key), other.styles.get(key)
+                if self_style is None:
                     logging.debug("%r missing in self.styles", key)
                     return False
-                elif ov is None:
+                elif other_style is None:
                     logging.debug("%r missing in other.styles", key)
                     return False
-                elif sv != ov:
-                    for k in sv.FIELDS:
-                        if getattr(sv, k) != getattr(ov, k): logging.debug("difference in field %r", k)
-                    logging.debug("style %r differs (self=%r, other=%r)", key, sv.as_dict(), ov.as_dict())
+                elif self_style != other_style:
+                    for k in self_style.FIELDS:
+                        if getattr(self_style, k) != getattr(other_style, k): logging.debug("difference in field %r", k)
+                    logging.debug("style %r differs (self=%r, other=%r)", key, self_style.as_dict(), other_style.as_dict())
                     return False
 
             if len(self) != len(other):
                 logging.debug("different # of subtitles (self=%d, other=%d)", len(self), len(other))
                 return False
 
-            for i, (se, oe) in enumerate(zip(self.events, other.events)):
-                if not se.equals(oe):
-                    for k in se.FIELDS:
-                        if getattr(se, k) != getattr(oe, k): logging.debug("difference in field %r", k)
-                    logging.debug("event %d differs (self=%r, other=%r)", i, se.as_dict(), oe.as_dict())
+            for i, (self_event, other_event) in enumerate(zip(self.events, other.events)):
+                if not self_event.equals(other_event):
+                    for k in self_event.FIELDS:
+                        if getattr(self_event, k) != getattr(other_event, k): logging.debug("difference in field %r", k)
+                    logging.debug("event %d differs (self=%r, other=%r)", i, self_event.as_dict(), other_event.as_dict())
                     return False
 
             return True
@@ -458,19 +471,52 @@ class SSAFile(MutableSequence):
         """Sort subtitles time-wise, in-place."""
         self.events.sort()
 
-    def __iter__(self) -> Iterable[SSAEvent]:
+    def __iter__(self) -> Iterator[SSAEvent]:
         return iter(self.events)
 
-    def __getitem__(self, item: int):
+    @overload
+    def __getitem__(self, item: int) -> SSAEvent:
         return self.events[item]
 
-    def __setitem__(self, key: int, value: SSAEvent):
-        if isinstance(value, SSAEvent):
-            self.events[key] = value
-        else:
-            raise TypeError("SSAFile.events must contain only SSAEvent objects")
+    @overload
+    def __getitem__(self, s: slice) -> List[SSAEvent]:
+        return self.events[s]
 
+    def __getitem__(self, item):
+        return self.events[item]
+
+    @overload
+    def __setitem__(self, key: int, value: SSAEvent):
+        pass
+
+    @overload
+    def __setitem__(self, keys: slice, values: Iterable[SSAEvent]):
+        pass
+
+    def __setitem__(self, key, value):
+        if isinstance(key, int):
+            if isinstance(value, SSAEvent):
+                self.events[key] = value
+            else:
+                raise TypeError("SSAFile.events must contain only SSAEvent objects")
+        elif isinstance(key, slice):
+            values = list(value)
+            if all(isinstance(v, SSAEvent) for v in values):
+                self.events[key] = values
+            else:
+                raise TypeError("SSAFile.events must contain only SSAEvent objects")
+        else:
+            raise TypeError("Bad key type")
+
+    @overload
     def __delitem__(self, key: int):
+        pass
+
+    @overload
+    def __delitem__(self, s: slice):
+        pass
+
+    def __delitem__(self, key):
         del self.events[key]
 
     def __len__(self):
