@@ -1,4 +1,8 @@
 import re
+import warnings
+from typing import List
+
+import pysubs2
 from .formatbase import FormatBase
 from .ssaevent import SSAEvent
 from .ssastyle import SSAStyle
@@ -17,11 +21,13 @@ class SubripFormat(FormatBase):
     @staticmethod
     def ms_to_timestamp(ms: int) -> str:
         """Convert ms to 'HH:MM:SS,mmm'"""
-        # XXX throw on overflow/underflow?
-        if ms < 0: ms = 0
-        if ms > MAX_REPRESENTABLE_TIME: ms = MAX_REPRESENTABLE_TIME
+        if ms < 0:
+            ms = 0
+        if ms > MAX_REPRESENTABLE_TIME:
+            warnings.warn("Overflow in SubRip timestamp, clamping to MAX_REPRESENTABLE_TIME", RuntimeWarning)
+            ms = MAX_REPRESENTABLE_TIME
         h, m, s, ms = ms_to_times(ms)
-        return "%02d:%02d:%02d,%03d" % (h, m, s, ms)
+        return f"{h:02d}:{m:02d}:{s:02d},{ms:03d}"
 
     @staticmethod
     def timestamp_to_ms(groups):
@@ -43,7 +49,7 @@ class SubripFormat(FormatBase):
                 return "srt"
 
     @classmethod
-    def from_file(cls, subs, fp, format_, keep_unknown_html_tags=False, **kwargs):
+    def from_file(cls, subs, fp, format_, keep_html_tags=False, keep_unknown_html_tags=False, **kwargs):
         """
         See :meth:`pysubs2.formats.FormatBase.from_file()`
 
@@ -52,10 +58,17 @@ class SubripFormat(FormatBase):
           - ``<i>``
           - ``<u>``
           - ``<s>``
+          - ``<b>``
 
         Keyword args:
-            keep_unknown_html_tags: If True, HTML tags other than i/u/s will be kept as-is.
-                Otherwise, they will be stripped from input.
+            keep_html_tags: If True, all HTML tags will be kept as-is instead of being
+                converted to SubStation tags (eg. you will get ``<i>example</i>`` instead of ``{\\i1}example{\\i0}``).
+                Setting this to True overrides the ``keep_unknown_html_tags`` option.
+            keep_unknown_html_tags: If True, supported HTML tags will be converted
+                to SubStation tags and any other HTML tags will be kept as-is
+                (eg. you would get ``<blink>example {\\i1}text{\\i0}</blink>``).
+                If False, these other HTML tags will be stripped from output
+                (in the previous example, you would get only ``example {\\i1}text{\\i0}``).
         """
         timestamps = [] # (start, end)
         following_lines = [] # contains lists of lines following each timestamp
@@ -81,13 +94,16 @@ class SubripFormat(FormatBase):
             # Handle the general case.
             s = "".join(lines).strip()
             s = re.sub(r"\n+ *\d+ *$", "", s) # strip number of next subtitle
-            s = re.sub(r"< *i *>", r"{\\i1}", s)
-            s = re.sub(r"< */ *i *>", r"{\\i0}", s)
-            s = re.sub(r"< *s *>", r"{\\s1}", s)
-            s = re.sub(r"< */ *s *>", r"{\\s0}", s)
-            s = re.sub(r"< *u *>", "{\\\\u1}", s) # not r" for Python 2.7 compat, triggers unicodeescape
-            s = re.sub(r"< */ *u *>", "{\\\\u0}", s)
-            if not keep_unknown_html_tags:
+            if not keep_html_tags:
+                s = re.sub(r"< *i *>", r"{\\i1}", s)
+                s = re.sub(r"< */ *i *>", r"{\\i0}", s)
+                s = re.sub(r"< *s *>", r"{\\s1}", s)
+                s = re.sub(r"< */ *s *>", r"{\\s0}", s)
+                s = re.sub(r"< *u *>", r"{\\u1}", s)
+                s = re.sub(r"< */ *u *>", r"{\\u0}", s)
+                s = re.sub(r"< *b *>", r"{\\b1}", s)
+                s = re.sub(r"< */ *b *>", r"{\\b0}", s)
+            if not (keep_html_tags or keep_unknown_html_tags):
                 s = re.sub(r"< */? *[a-zA-Z][^>]*>", "", s) # strip other HTML tags
             s = re.sub(r"\n", r"\\N", s) # convert newlines
             return s
@@ -129,15 +145,15 @@ class SubripFormat(FormatBase):
             else:
                 for fragment, sty in parse_tags(text, style, subs.styles):
                     if apply_styles:
-                        if sty.italic: fragment = "<i>%s</i>" % fragment
-                        if sty.underline: fragment = "<u>%s</u>" % fragment
-                        if sty.strikeout: fragment = "<s>%s</s>" % fragment
+                        if sty.italic: fragment = f"<i>{fragment}</i>"
+                        if sty.underline: fragment = f"<u>{fragment}</u>"
+                        if sty.strikeout: fragment = f"<s>{fragment}</s>"
                     if sty.drawing: raise ContentNotUsable
                     body.append(fragment)
 
             return re.sub("\n+", "\n", "".join(body).strip())
 
-        visible_lines = (line for line in subs if not line.is_comment)
+        visible_lines = cls._get_visible_lines(subs)
 
         lineno = 1
         for line in visible_lines:
@@ -148,7 +164,12 @@ class SubripFormat(FormatBase):
             except ContentNotUsable:
                 continue
 
-            print("%d" % lineno, file=fp) # Python 2.7 compat
+            print(lineno, file=fp)
             print(start, "-->", end, file=fp)
             print(text, end="\n\n", file=fp)
             lineno += 1
+
+    @classmethod
+    def _get_visible_lines(cls, subs: "pysubs2.SSAFile") -> List["pysubs2.SSAEvent"]:
+        visible_lines = [line for line in subs if not line.is_comment]
+        return visible_lines
