@@ -4,7 +4,7 @@ from typing import Optional, List, Tuple, Sequence
 from pysubs2.common import IntOrFloat
 
 #: Pattern that matches both SubStation and SubRip timestamps.
-TIMESTAMP = re.compile(r"(\d{1,2}):(\d{2}):(\d{2})[.,](\d{2,3})")
+TIMESTAMP = re.compile(r"(\d{1,2}):(\d{1,2}):(\d{1,2})[.,](\d{1,3})")
 
 #: Pattern that matches H:MM:SS or HH:MM:SS timestamps.
 TIMESTAMP_SHORT = re.compile(r"(\d{1,2}):(\d{2}):(\d{2})")
@@ -98,7 +98,7 @@ def frames_to_ms(frames: int, fps: float) -> int:
     
     """
     if fps <= 0:
-        raise ValueError("Framerate must be positive number (%f)." % fps)
+        raise ValueError(f"Framerate must be a positive number ({fps}).")
 
     return int(round(frames * (1000 / fps)))
 
@@ -119,7 +119,7 @@ def ms_to_frames(ms: IntOrFloat, fps: float) -> int:
     
     """
     if fps <= 0:
-        raise ValueError("Framerate must be positive number (%f)." % fps)
+        raise ValueError(f"Framerate must be a positive number ({fps}).")
 
     return int(round((ms / 1000) * fps))
 
